==> hubzero-submit-distributor_2.5.6.orig.tar.gz: hubzero-submit-distributor-2.5.6/hubzero/submit/ToolsInfo.py <==
# @package      hubzero-submit-distributor
# @file         ToolsInfo.py
# @author       Steven Clark <clarks@purdue.edu>
# @copyright    Copyright (c) 2012-2013 HUBzero Foundation, LLC.
# @license      http://www.gnu.org/licenses/lgpl-3.0.html LGPLv3
#
# Copyright (c) 2012-2013 HUBzero Foundation, LLC.
#
# This file is part of: The HUBzero(R) Platform for Scientific Collaboration
#
# The HUBzero(R) Platform for Scientific Collaboration (HUBzero) is free
# software: you can redistribute it and/or modify it under the terms of
# the GNU Lesser General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any
# later version.
#
# HUBzero is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# HUBzero is a registered trademark of HUBzero Foundation, LLC.
#

import os.path
import re
import random

from hubzero.submit.LogMessage      import logJobId as log
from hubzero.submit.GroupMembership import isGroupMember

class ToolsInfo:
   def __init__(self,
                infoDirectory,
                toolsFile):
      self.tools      = {}

      toolPattern     = re.compile('(\s*\[)([^\s]*)(]\s*)')
      keyValuePattern = re.compile('( *)(\w*)( *= *)(.*[^\s$])( *)')
      commentPattern  = re.compile('\s*#.*')
      toolName        = ""

      toolsPath = os.path.join(infoDirectory,toolsFile)
      if os.path.exists(toolsPath):
         fpInfo = open(toolsPath,'r')
         if fpInfo:
            eof = False
            while not eof:
               record = fpInfo.readline()
               if record != "":
                  record = commentPattern.sub("",record)
                  if   toolPattern.match(record):
                     toolName = toolPattern.match(record).group(2)
                     if not toolName in self.tools:
                        self.tools[toolName] = []
                     toolGroupIndex = len(self.tools[toolName])
                     self.tools[toolName].append({'destinations':[],
                                                  'executablePath':'',
                                                  'isMPI':'',
                                                  'restrictedToUsers':[],
                                                  'restrictedToGroups':[],
                                                  'environment':[],
                                                  'remoteManager':'',
                                                  'state':'enabled'
                                                 })
                  elif keyValuePattern.match(record):
                     key,value = keyValuePattern.match(record).group(2,4)
                     if key in self.tools[toolName][toolGroupIndex]:
                        if   isinstance(self.tools[toolName][toolGroupIndex][key],list):
                           self.tools[toolName][toolGroupIndex][key] = [e.strip() for e in value.split(',')]
                        elif isinstance(self.tools[toolName][toolGroupIndex][key],dict):
                           dictKey,dictValue = value.split(':')
                           self.tools[toolName][toolGroupIndex][key][dictKey.strip()] = dictValue.strip()
                        elif isinstance(self.tools[toolName][toolGroupIndex][key],bool):
                           self.tools[toolName][toolGroupIndex][key] = bool(value.lower() == 'true')
                        else:
                           self.tools[toolName][toolGroupIndex][key] = value
                     else:
                        log("Undefined tool key %s %s in tool %s" % (key,value,toolName))
               else:
                  eof = True
            fpInfo.close()

         for toolName in self.tools:
            markedForDeletion = []
            for toolGroupIndex in xrange(len(self.tools[toolName])):
               state = self.tools[toolName][toolGroupIndex]['state']
               if state == 'disabled':
                  markedForDeletion.append(toolGroupIndex)
            markedForDeletion.reverse()
            for toolGroupIndex in markedForDeletion:
               del self.tools[toolName][toolGroupIndex]
            del markedForDeletion
      else:
         log("Tools configuration file %s is missing" % (toolsPath))


   def applyUserRestriction(self,
                            userName):
      for toolName in self.tools:
         markedForDeletion = []
         for toolGroupIndex in xrange(len(self.tools[toolName])):
            restrictedToUsers = self.tools[toolName][toolGroupIndex]['restrictedToUsers']
            if len(restrictedToUsers) > 0:
               if userName in restrictedToUsers:
                  userOK = True
               else:
                  userOK = False
               if not userOK:
                  markedForDeletion.append(toolGroupIndex)
         markedForDeletion.reverse()
         for toolGroupIndex in markedForDeletion:
            del self.tools[toolName][toolGroupIndex]
         del markedForDeletion


   def applyGroupRestriction(self):
      for toolName in self.tools:
         markedForDeletion = []
         for toolGroupIndex in xrange(len(self.tools[toolName])):
            restrictedToGroups = self.tools[toolName][toolGroupIndex]['restrictedToGroups']
            if len(restrictedToGroups) > 0:
               groupOK = False
               for restrictedToGroup in restrictedToGroups:
                  if isGroupMember(restrictedToGroup):
                     groupOK = True
                     break
               if not groupOK:
                  markedForDeletion.append(toolGroupIndex)
         markedForDeletion.reverse()
         for toolGroupIndex in markedForDeletion:
            del self.tools[toolName][toolGroupIndex]
         del markedForDeletion


   def purgeDisabledSites(self,
                          enabledSites):
      for toolName in self.tools:
         toolInfo = self.tools[toolName]
         for toolGroupIndex in xrange(len(toolInfo)):
            markedForDeletion = []
            for toolDestination in toolInfo[toolGroupIndex]['destinations']:
               if not toolDestination in enabledSites:
                  markedForDeletion.append(toolDestination)
            markedForDeletion.reverse()
            for toolDestination in markedForDeletion:
               self.tools[toolName][toolGroupIndex]['destinations'].remove(toolDestination)
            del markedForDeletion
            if len(self.tools[toolName][toolGroupIndex]['destinations']) == 0:
               self.tools[toolName][toolGroupIndex]['state'] = 'sitesUnavailable'


   def purgeNonPegasusSites(self,
                            nonPegasusSites):
      for toolName in self.tools:
         toolInfo = self.tools[toolName]
         for toolGroupIndex in xrange(len(toolInfo)):
            markedForDeletion = []
            for toolDestination in toolInfo[toolGroupIndex]['destinations']:
               if toolDestination in nonPegasusSites:
                  markedForDeletion.append(toolDestination)
            markedForDeletion.reverse()
            for toolDestination in markedForDeletion:
               self.tools[toolName][toolGroupIndex]['destinations'].remove(toolDestination)
            del markedForDeletion
            if len(self.tools[toolName][toolGroupIndex]['destinations']) == 0:
               self.tools[toolName][toolGroupIndex]['state'] = 'sitesUnavailable'


   def purgePegasusSites(self,
                         pegasusSites):
      for toolName in self.tools:
         toolInfo = self.tools[toolName]
         for toolGroupIndex in xrange(len(toolInfo)):
            markedForDeletion = []
            nNonPegasusSites = 0
            for toolDestination in toolInfo[toolGroupIndex]['destinations']:
               if toolDestination in pegasusSites:
                  markedForDeletion.append(toolDestination)
               else:
                  nNonPegasusSites += 1
            if nNonPegasusSites > 0 and len(markedForDeletion) > 0:
               markedForDeletion.reverse()
               for toolDestination in markedForDeletion:
                  self.tools[toolName][toolGroupIndex]['destinations'].remove(toolDestination)
               if len(self.tools[toolName][toolGroupIndex]['destinations']) == 0:
                  self.tools[toolName][toolGroupIndex]['state'] = 'sitesUnavailable'
            del markedForDeletion


   def isExecutableTool(self,
                        executable):
      return(executable in self.tools)


   def isPermissionGranted(self,
                           executable):
      permissionGranted = False
      if len(self.tools[executable]) > 0:
         permissionGranted = True

      return(permissionGranted)


   def getDefaultToolInfo(self):
      toolInfo = {}
      toolInfo['destinations'] = []
      toolInfo['executablePath']   = ""
      toolInfo['environment']      = []
      toolInfo['remoteManager']    = ""

      return(toolInfo)


   def selectTool(self,
                  executable,
                  userDestinations,
                  remoteProbeMonitor):
      selectedToolInfo = {}
      selectedToolInfo['destinations']   = []
      selectedToolInfo['executablePath'] = ""
      selectedToolInfo['environment']    = []
      selectedToolInfo['remoteManager']  = ""

      toolInfo = self.tools[executable]
      if len(userDestinations) == 0:
         toolGroupIndexes = range(len(toolInfo))
         while len(toolGroupIndexes) > 0 and len(selectedToolInfo['destinations']) == 0:
            toolGroupIndex = random.choice(toolGroupIndexes)
            for toolDestination in toolInfo[toolGroupIndex]['destinations']:
               selectedToolInfo['destinations'].append(toolDestination)
            if remoteProbeMonitor:
               remoteProbeMonitor.purgeOfflineSites(selectedToolInfo['destinations'])
            if len(selectedToolInfo['destinations']) == 0:
               toolGroupIndexes.remove(toolGroupIndex)
      else:
         toolGroupIndex = -1
         for userDestination in userDestinations:
            for groupIndex in xrange(len(toolInfo)):
               if userDestination in toolInfo[groupIndex]['destinations']:
                  toolGroupIndex = groupIndex
                  break
            if toolGroupIndex != -1:
               break

         if toolGroupIndex != -1:
            for toolDestination in toolInfo[toolGroupIndex]['destinations']:
               if toolDestination in userDestinations:
                  selectedToolInfo['destinations'].append(toolDestination)
         else:
            log("Invalid destination(s) specified: %s" % (userDestinations))

      if len(selectedToolInfo['destinations']) > 0:
         selectedToolInfo['executablePath']   = toolInfo[toolGroupIndex]['executablePath']
         if 'environment' in toolInfo[toolGroupIndex]:
            selectedToolInfo['environment']   = toolInfo[toolGroupIndex]['environment']
         if 'remoteManager' in toolInfo[toolGroupIndex]:
            selectedToolInfo['remoteManager'] = toolInfo[toolGroupIndex]['remoteManager']

      return(selectedToolInfo)


   def getToolNames(self):
      toolNames = []
      for toolName in self.tools:
         if self.isPermissionGranted(toolName):
            toolNames.append(toolName)

      return(toolNames)

==> hubzero-submit-distributor_2.7.2.4.orig.tar.gz: hubzero-submit-distributor-2.7.2.4/hubzero/submit/ToolsInfo.py <==
# @package      hubzero-submit-distributor
# @file         ToolsInfo.py
# @author       Steven Clark <clarks@purdue.edu>
# @copyright    Copyright (c) 2012-2014 HUBzero Foundation, LLC.
# @license      http://www.gnu.org/licenses/lgpl-3.0.html LGPLv3
#
# Copyright (c) 2012-2014 HUBzero Foundation, LLC.
#
# This file is part of: The HUBzero(R) Platform for Scientific Collaboration
#
# The HUBzero(R) Platform for Scientific Collaboration (HUBzero) is free
# software: you can redistribute it and/or modify it under the terms of
# the GNU Lesser General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any
# later version.
#
# HUBzero is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# HUBzero is a registered trademark of HUBzero Foundation, LLC.
#

import os.path
import re
import random
import logging

from hubzero.submit.LogMessage      import getLogJobIdMessage as getLogMessage
from hubzero.submit.GroupMembership import isGroupMember

class ToolsInfo:
   def __init__(self,
                infoDirectory,
                toolsFile):
      self.logger     = logging.getLogger(__name__)
      self.tools      = {}

      toolPattern     = re.compile('(\s*\[)([^\s]*)(]\s*)')
      keyValuePattern = re.compile('( *)(\w*)( *= *)(.*[^\s$])( *)')
      commentPattern  = re.compile('\s*#.*')
      toolName        = ""

      toolsPath = os.path.join(infoDirectory,toolsFile)
      if os.path.exists(toolsPath):
         fpInfo = open(toolsPath,'r')
         if fpInfo:
            eof = False
            while not eof:
               record = fpInfo.readline()
               if record != "":
                  record = commentPattern.sub("",record)
                  if   toolPattern.match(record):
                     toolName = toolPattern.match(record).group(2)
                     if not toolName in self.tools:
                        self.tools[toolName] = []
                     toolGroupIndex = len(self.tools[toolName])
                     self.tools[toolName].append({'destinations':[],
                                                  'executablePath':'',
                                                  'isMPI':'',
                                                  'restrictedToUsers':[],
                                                  'restrictedToGroups':[],
                                                  'environment':[],
                                                  'remoteManager':'',
                                                  'state':'enabled'
                                                 })
                  elif keyValuePattern.match(record):
                     key,value = keyValuePattern.match(record).group(2,4)
                     if key in self.tools[toolName][toolGroupIndex]:
                        if   isinstance(self.tools[toolName][toolGroupIndex][key],list):
                           self.tools[toolName][toolGroupIndex][key] = [e.strip() for e in value.split(',')]
                        elif isinstance(self.tools[toolName][toolGroupIndex][key],dict):
                           dictKey,dictValue = value.split(':')
                           self.tools[toolName][toolGroupIndex][key][dictKey.strip()] = dictValue.strip()
                        elif isinstance(self.tools[toolName][toolGroupIndex][key],bool):
                           self.tools[toolName][toolGroupIndex][key] = bool(value.lower() == 'true')
                        else:
                           self.tools[toolName][toolGroupIndex][key] = value
                     else:
                        self.logger.log(logging.WARNING,getLogMessage("Undefined tool key %s %s in tool %s" % (key,value,toolName)))
               else:
                  eof = True
            fpInfo.close()

         for toolName in self.tools:
            markedForDeletion = []
            for toolGroupIndex in xrange(len(self.tools[toolName])):
               state = self.tools[toolName][toolGroupIndex]['state']
               if state == 'disabled':
                  markedForDeletion.append(toolGroupIndex)
            markedForDeletion.reverse()
            for toolGroupIndex in markedForDeletion:
               del self.tools[toolName][toolGroupIndex]
            del markedForDeletion
      else:
         self.logger.log(logging.ERROR,getLogMessage("Tools configuration file %s is missing" % (toolsPath)))


   def applyUserRestriction(self,
                            userName):
      for toolName in self.tools:
         markedForDeletion = []
         for toolGroupIndex in xrange(len(self.tools[toolName])):
            restrictedToUsers = self.tools[toolName][toolGroupIndex]['restrictedToUsers']
            if len(restrictedToUsers) > 0:
               if userName in restrictedToUsers:
                  userOK = True
               else:
                  userOK = False
               if not userOK:
                  markedForDeletion.append(toolGroupIndex)
         markedForDeletion.reverse()
         for toolGroupIndex in markedForDeletion:
            del self.tools[toolName][toolGroupIndex]
         del markedForDeletion


   def applyGroupRestriction(self):
      for toolName in self.tools:
         markedForDeletion = []
         for toolGroupIndex in xrange(len(self.tools[toolName])):
            restrictedToGroups = self.tools[toolName][toolGroupIndex]['restrictedToGroups']
            if len(restrictedToGroups) > 0:
               groupOK = False
               for restrictedToGroup in restrictedToGroups:
                  if isGroupMember(restrictedToGroup):
                     groupOK = True
                     break
               if not groupOK:
                  markedForDeletion.append(toolGroupIndex)
         markedForDeletion.reverse()
         for toolGroupIndex in markedForDeletion:
            del self.tools[toolName][toolGroupIndex]
         del markedForDeletion


   def purgeDisabledSites(self,
                          enabledSites):
      for toolName in self.tools:
         toolInfo = self.tools[toolName]
         for toolGroupIndex in xrange(len(toolInfo)):
            markedForDeletion = []
            for toolDestination in toolInfo[toolGroupIndex]['destinations']:
               if not toolDestination in enabledSites:
                  markedForDeletion.append(toolDestination)
            markedForDeletion.reverse()
            for toolDestination in markedForDeletion:
               self.tools[toolName][toolGroupIndex]['destinations'].remove(toolDestination)
            del markedForDeletion
            if len(self.tools[toolName][toolGroupIndex]['destinations']) == 0:
               self.tools[toolName][toolGroupIndex]['state'] = 'sitesUnavailable'


   def purgeNonPegasusSites(self,
                            nonPegasusSites):
      for toolName in self.tools:
         toolInfo = self.tools[toolName]
         for toolGroupIndex in xrange(len(toolInfo)):
            markedForDeletion = []
            for toolDestination in toolInfo[toolGroupIndex]['destinations']:
               if toolDestination in nonPegasusSites:
                  markedForDeletion.append(toolDestination)
            markedForDeletion.reverse()
            for toolDestination in markedForDeletion:
               self.tools[toolName][toolGroupIndex]['destinations'].remove(toolDestination)
            del markedForDeletion
            if len(self.tools[toolName][toolGroupIndex]['destinations']) == 0:
               self.tools[toolName][toolGroupIndex]['state'] = 'sitesUnavailable'


   def purgePegasusSites(self,
                         pegasusSites):
      for toolName in self.tools:
         toolInfo = self.tools[toolName]
         for toolGroupIndex in xrange(len(toolInfo)):
            markedForDeletion = []
            nNonPegasusSites = 0
            for toolDestination in toolInfo[toolGroupIndex]['destinations']:
               if toolDestination in pegasusSites:
                  markedForDeletion.append(toolDestination)
               else:
                  nNonPegasusSites += 1
            if nNonPegasusSites > 0 and len(markedForDeletion) > 0:
               markedForDeletion.reverse()
               for toolDestination in markedForDeletion:
                  self.tools[toolName][toolGroupIndex]['destinations'].remove(toolDestination)
               if len(self.tools[toolName][toolGroupIndex]['destinations']) == 0:
                  self.tools[toolName][toolGroupIndex]['state'] = 'sitesUnavailable'
            del markedForDeletion


   def isExecutableTool(self,
                        executable):
      return(executable in self.tools)


   def isPermissionGranted(self,
                           executable):
      permissionGranted = False
      if len(self.tools[executable]) > 0:
         permissionGranted = True

      return(permissionGranted)


   def getDefaultToolInfo(self):
      toolInfo = {}
      toolInfo['destinations']   = []
      toolInfo['executablePath'] = ""
      toolInfo['environment']    = []
      toolInfo['remoteManager']  = ""

      return(toolInfo)


   def selectTool(self,
                  executable,
                  userDestinations,
                  remoteProbeMonitor):
      selectedToolInfo = {}
      selectedToolInfo['destinations']   = []
      selectedToolInfo['executablePath'] = ""
      selectedToolInfo['environment']    = []
      selectedToolInfo['remoteManager']  = ""

      toolInfo = self.tools[executable]
      if len(userDestinations) == 0:
         toolGroupIndexes = range(len(toolInfo))
         while len(toolGroupIndexes) > 0 and len(selectedToolInfo['destinations']) == 0:
            toolGroupIndex = random.choice(toolGroupIndexes)
            for toolDestination in toolInfo[toolGroupIndex]['destinations']:
               selectedToolInfo['destinations'].append(toolDestination)
            if remoteProbeMonitor:
               remoteProbeMonitor.purgeOfflineSites(selectedToolInfo['destinations'])
            if len(selectedToolInfo['destinations']) == 0:
               toolGroupIndexes.remove(toolGroupIndex)
      else:
         toolGroupIndex = -1
         for userDestination in userDestinations:
            for groupIndex in xrange(len(toolInfo)):
               if userDestination in toolInfo[groupIndex]['destinations']:
                  toolGroupIndex = groupIndex
                  break
            if toolGroupIndex != -1:
               break

         if toolGroupIndex != -1:
            for toolDestination in toolInfo[toolGroupIndex]['destinations']:
               if toolDestination in userDestinations:
                  selectedToolInfo['destinations'].append(toolDestination)
         else:
            self.logger.log(logging.ERROR,getLogMessage("Invalid destination(s) specified: %s" % (userDestinations)))

      if len(selectedToolInfo['destinations']) > 0:
         selectedToolInfo['executablePath']   = toolInfo[toolGroupIndex]['executablePath']
         if 'environment' in toolInfo[toolGroupIndex]:
            selectedToolInfo['environment']   = toolInfo[toolGroupIndex]['environment']
         if 'remoteManager' in toolInfo[toolGroupIndex]:
            selectedToolInfo['remoteManager'] = toolInfo[toolGroupIndex]['remoteManager']

      return(selectedToolInfo)


   def getToolNames(self):
      toolNames = []
      for toolName in self.tools:
         if self.isPermissionGranted(toolName):
            toolNames.append(toolName)

      return(toolNames)


   def getToolDestinations(self,
                           toolName):
      destinations = []

      toolInfo = self.tools[toolName]
      for toolGroupIndex in range(len(toolInfo)):
         for toolDestination in toolInfo[toolGroupIndex]['destinations']:
            if not toolDestination in destinations:
               destinations.append(toolDestination)

      return(destinations)
